//**********************************************************************
//Written by John Tobler jtobler@us.ibm.com 9/02 for IBM Use Only
//These function are used to convert stck values to human readable dates
//and times.  This also supports caluculating the stck value of a human
//readable date/time.  Since a range of stck values represent a given
//msec in time, the range of stck values will be presented for this type
//of conversion.  Enjoy...
//**********************************************************************

//Constants
var YEAR = 0x1CAE8C13E000000;
var DAY  = 0x00141DD76000000;
var HOUR = 0x0000D693A400000;
var MIN  = 0x000003938700000;
var SEC  = 0x0000000F4240000;
var MSEC = 0x0000000003E8000;
var OVERFLOW = "Overflow";
var INVALID = "Invalid Hex";
var BIG_VAL = "FFFFFFFFFFFFFFFF";
var BASE_YEAR = 1900;
var DAYS_YEAR = 365;
var DAYS_LEAPYEAR = 366;
var FEB = 2;
var days = new Array(31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);


//**********************************************************************
//Core methods: elapToDw, dwToElap, stckToDate
//Description:  These are executed directly via convert buttom click
//              events on the form.
//**********************************************************************

//Convert an elapsed time to a double word value
function elapToDW(){
   
   //validate the entry fields
   var f = document["converter"].elements;
   var oETStckRange = document.getElementById("ETStckRange");
   if (!isValidInt(f["ETD"].value,0,0)){
      f["ETD"].value = "?"
      return;
   }
   if (!isValidInt(f["ETHR"].value,23,0)){
      f["ETHR"].value = "?"
      return;
   }
   if (!isValidInt(f["ETMIN"].value,59,0)){
      f["ETMIN"].value = "?"
      return;
   }
   if (!isValidInt(f["ETSEC"].value,59,0)){
      f["ETSEC"].value = "?"
      return;
   }
   if (!isValidInt(f["ETMSEC"].value,0,0)){
      f["ETMSEC"].value = "?"
      return;
   }
   
   //get the decimal representation of the
   //specified elapsed time
   var lrange = 0;
   var hrange = 0;
   f["ETD"].value = addPad(f["ETD"].value,1);
   lrange += f["ETD"].value * DAY;
   f["ETHR"].value = addPad(f["ETHR"].value,2);
   lrange += f["ETHR"].value * HOUR;
   f["ETMIN"].value = addPad(f["ETMIN"].value,2);
   lrange += f["ETMIN"].value * MIN;
   f["ETSEC"].value = addPad(f["ETSEC"].value,2);
   lrange += f["ETSEC"].value * SEC;
   f["ETMSEC"].value = addPad(f["ETMSEC"].value,3);
   lrange += f["ETMSEC"].value * MSEC;
   hrange = lrange + MSEC;

   //convert dec to base 16
   f["ETStck"].value = toHex(lrange);
   if (f["ETStck"].value == OVERFLOW){
     oETStckRange.innerHTML = "";
     return;
   }
   hrange = toHex(hrange);
   
   //the high range will either be 8000 or 0000 for the
   //last 2 bytes.  It should be 1 less than this value
   //but we don't have precision over the last 15 bits
   if (hrange == OVERFLOW){
     oETStckRange.innerHTML = "to " + BIG_VAL;
   }
   else if (hrange.substring(hrange.length - 4, hrange.length) == "8000"){
     oETStckRange.innerHTML = "to " + hrange.substring(0,hrange.length - 4) +
                              "7FFF";
   }
   else{
     var i = 5;
     var append = "FFFF";
     while(hrange.charAt(hrange.length - i) == '0'){
       append = append + "F";
       i++;    
     } 
     oETStckRange.innerHTML = "to " + hrange.substring(0,hrange.length - i) +
                               toHex("0x" + hrange.charAt(hrange.length - i) - 1) + 
                               append;
   }
}

//Convert a double word to elapsed time
function dwToElap(){

  var f = document["converter"].elements;
  var oETStckRange = document.getElementById("ETStckRange");
  oETStckRange.innerHTML = "";
  var dw = f["ETStck"].value;
  if (dw.length == 0){
    return;
  }

  //validate the input...if not valid,
  //send a message to the form
  if (!isValidHex(dw)){

    f["ETStck"].value = INVALID;
    f["ETD"].value = "";
    f["ETHR"].value = "";
    f["ETMIN"].value = "";
    f["ETSEC"].value = "";
    f["ETMSEC"].value = "";
    oETStckRange.innerHTML = "";
    return;
  }

  //The last 15 bits of very large numbers in Javascript
  //are suspect and there precision may not be maintained.
  //Thus we chop off the last 16 bits and will add it back
  //in at the end of the conversion.
  var additional = "0x" + dw.substring(dw.length - 4, dw.length);
  dw = "0x" + dw.substring(0,dw.length - 4) + "0000";
 
  //divide by the highest unit of time to the lowest
  //until we have the number of days, hours, min, sec, and msec
  var etd = floatToDec(dw / DAY);
  dw = dw % DAY;
  var ethr = floatToDec(dw / HOUR);
  dw = dw % HOUR;
  var etmin = floatToDec(dw / MIN);
  dw = dw % MIN;
  var etsec = floatToDec(dw / SEC);
  dw = dw % SEC;

  //we can add back in the additional
  dw = dw + +additional;
  var etmsec = floatToDec(dw / MSEC);

  //print the answer
  f["ETD"].value = etd;
  f["ETHR"].value = addPad(ethr,2);
  f["ETMIN"].value = addPad(etmin,2);
  f["ETSEC"].value = addPad(etsec,2);
  f["ETMSEC"].value = addPad(etmsec,3);

}

//Convert a stck date to human readable and a Julian date
function stckToDate(){

  var f = document["converter"].elements;
  var oDTStckRange = document.getElementById("DTStckRange");
  oDTStckRange.innerHTML = "";
  var stckVal = f["DTStck"].value;
  if (stckVal.length == 0){
    return;
  }

  //validate the input...if not valid
  //print a message on the form
  if (!isValidHex(stckVal)){

    //clear everything and send a message
    f["DTStck"].value = INVALID;
    f["DTMM"].value = "";
    f["DTDD"].value = "";
    f["DTYYYY"].value = "";
    f["DTJYYYY"].value = "";
    f["DTJD"].value = "";
    f["DTHR"].value = "";
    f["DTMIN"].value = "";
    f["DTSEC"].value = "";
    f["DTMSEC"].value = "";
    oDTStckRange.innerHTML = "";
    return;
  }

  //The last 15 bits of very large numbers in Javascript
  //are suspect and there precision may not be maintained.
  //Thus we chop off the last 16 bits and will add it back
  //in at the end of the conversion.
  var additional = "0x" + stckVal.substring(stckVal.length - 4, stckVal.length);
  stckVal = "0x" + stckVal.substring(0,stckVal.length - 4) + "0000";

  //get the year...
  var dtyy = 0;
  while (+stckVal >= YEAR){
    stckVal = stckVal - YEAR;
 
    //check for leap years
    if (isLeapYear(dtyy + BASE_YEAR)){
      stckVal = stckVal - DAY;
    }
    dtyy++;
  }
  //the clock started in 1900 so we add on the
  //base year
  dtyy += BASE_YEAR;

  //get the Julian date...we are on the day after
  //the number of full days that have passed
  var dtjd = floatToDec((stckVal / DAY) + 1);
  stckVal = stckVal % DAY;

  var dthr = floatToDec(stckVal / HOUR);
  stckVal = stckVal % HOUR;
  var dtmin = floatToDec(stckVal / MIN);
  stckVal = stckVal % MIN;
  var dtsec = floatToDec(stckVal / SEC);
  stckVal = stckVal % SEC;

  //we can add back in the additional
  stckVal = stckVal + +additional;
  var dtmsec = floatToDec(stckVal / MSEC);
   
  //get the month
  var dtmm = 1;
  var dtot = 0;
  var daysNextMonth = days[0];
  while(dtot + daysNextMonth < dtjd){
    dtot += daysNextMonth;
    daysNextMonth = days[dtmm];
    if ((dtmm + 1) == FEB && isLeapYear(dtyy)){
      daysNextMonth++;    
    }
    dtmm++; 
  }

  //display the results
  f["DTMM"].value = addPad(dtmm,2);
  f["DTDD"].value = addPad(dtjd - dtot,2);
  f["DTYYYY"].value = dtyy;
  f["DTJYYYY"].value = dtyy;
  f["DTJD"].value = dtjd;
  f["DTHR"].value = addPad(dthr,2);
  f["DTMIN"].value = addPad(dtmin,2);
  f["DTSEC"].value = addPad(dtsec,2);
  f["DTMSEC"].value = addPad(dtmsec,3);                                                                                                                                                                                                                                                                                                  		
}


//Convert a Julian Date to a stck value range
function jToStck(){

   //validate the entry fields
   var f = document["converter"].elements;
   var oDTStckRange = document.getElementById("DTStckRange");
   if (!isValidInt(f["DTJYYYY"].value,2042,1900)){
      f["DTJYYYY"].value = "?"
      return;
   }
   var dayLimit = DAYS_YEAR;
   if (isLeapYear(f["DTJYYYY"].value)){
     dayLimit = DAYS_LEAPYEAR;
   }
   if (!isValidInt(f["DTJD"].value,dayLimit,1)){
      f["DTJD"].value = "?"
      return;
   }
   if (!isValidInt(f["DTMIN"].value,59,0)){
      f["DTMIN"].value = "?"
      return;
   }
   if (!isValidInt(f["DTSEC"].value,59,0)){
      f["DTSEC"].value = "?"
      return;
   }
   if (!isValidInt(f["DTMSEC"].value,0,0)){
      f["DTMSEC"].value = "?"
      return;
   }
   
   //get the number of days that have passed
   var lrange = 0;
   var daysPassed = yearToDays(f["DTJYYYY"].value);
   daysPassed += f["DTJD"].value - 1;
   lrange += daysPassed * DAY;

   f["DTHR"].value = addPad(f["DTHR"].value,2);
   lrange += f["DTHR"].value * HOUR;
   f["DTMIN"].value = addPad(f["DTMIN"].value,2);
   lrange += f["DTMIN"].value * MIN;
   f["DTSEC"].value = addPad(f["DTSEC"].value,2);
   lrange += f["DTSEC"].value * SEC;
   f["DTMSEC"].value = addPad(f["DTMSEC"].value,3);
   lrange += f["DTMSEC"].value * MSEC;
   
   //print the info
   printDTRange(lrange);

   //print the mm/dd/yyyy information
   var dtmm = 1;
   var dtot = 0;
   var daysNextMonth = days[0];
   while(dtot + daysNextMonth < f["DTJD"].value){
     dtot += daysNextMonth;
     daysNextMonth = days[dtmm];
     if ((dtmm + 1) == FEB && isLeapYear(f["DTJYYYY"].value)){
       daysNextMonth++;    
     }
     dtmm++; 
   }	

   //display the results
   f["DTMM"].value = addPad(dtmm,2);
   f["DTDD"].value = addPad(f["DTJD"].value - dtot,2);
   f["DTYYYY"].value = f["DTJYYYY"].value;
   
}

//Convert a mm/dd/yyy date to a stck value range
function dateToStck(){

   //validate the entry fields
   var f = document["converter"].elements;
   var oDTStckRange = document.getElementById("DTStckRange");
   if (!isValidInt(f["DTMM"].value,12,1)){
      f["DTMM"].value = "?"
      return;
   }
   if (!isValidInt(f["DTYYYY"].value,2042,1900)){
      f["DTYYYY"].value = "?"
      return;
   }
   var dayLimit = days[f["DTMM"].value - 1];
   if (f["DTMM"].value == FEB && isLeapYear(f["DTYYYY"].value)){
     dayLimit++;
   }
   if (!isValidInt(f["DTDD"].value,dayLimit,1)){
      f["DTDD"].value = "?"
      return;
   }
   if (!isValidInt(f["DTMIN"].value,59,0)){
      f["DTMIN"].value = "?"
      return;
   }
   if (!isValidInt(f["DTSEC"].value,59,0)){
      f["DTSEC"].value = "?"
      return;
   }
   if (!isValidInt(f["DTMSEC"].value,0,0)){
      f["DTMSEC"].value = "?"
      return;
   }
   //get the number of days that have passed
   var lrange = 0;
   lrange += (yearToDays(f["DTYYYY"].value) * DAY);

   //now add the number of days that have passed in THIS year
   var daysThisYear = 0;
   f["DTMM"].value = addPad(f["DTMM"].value,2);
   f["DTDD"].value = addPad(f["DTDD"].value,2);
   for (i = 1; i < f["DTMM"].value; i++){
     if (i == FEB && isLeapYear(f["DTYYYY"].value)){
       daysThisYear++;
     }
     daysThisYear += days[i - 1];
   }
   daysThisYear += f["DTDD"].value - 1;
   lrange += daysThisYear * DAY; 

   f["DTHR"].value = addPad(f["DTHR"].value,2);
   lrange += f["DTHR"].value * HOUR;
   f["DTMIN"].value = addPad(f["DTMIN"].value,2);
   lrange += f["DTMIN"].value * MIN;
   f["DTSEC"].value = addPad(f["DTSEC"].value,2);
   lrange += f["DTSEC"].value * SEC;
   f["DTMSEC"].value = addPad(f["DTMSEC"].value,3);
   lrange += f["DTMSEC"].value * MSEC;
   
   //print the info
   printDTRange(lrange);
   
   //print the Julian Date Information
   f["DTJYYYY"].value = f["DTYYYY"].value;
   f["DTJD"].value = daysThisYear + 1; 

}


//**********************************************************************
//Utility methods: floatToDec, addPad, isLeapYear, isValidHex, toHex
//                 toHexVal, isValidInt, printDTRange, yearToDays
//Description:     These are used by the core methods.
//**********************************************************************



//The fucntion is used by the Julian and MM/DD/YYYY converted to display
//the range of stck values
function printDTRange(lrange){

  var f = document["converter"].elements;
  var oDTStckRange = document.getElementById("DTStckRange");
  var hrange = lrange + MSEC;

  //convert dec to base 16
  f["DTStck"].value = toHex(lrange);
  if (f["DTStck"].value == OVERFLOW){
     oDTStckRange.innerHTML = "";
     return;
  }
  hrange = toHex(hrange);
   
  //the high range will either be 8000 or 0000 for the
  //last 2 bytes.  It should be 1 less than this value
  //but we don't have precision over the last 15 bits
  if (hrange == OVERFLOW){
    oDTStckRange.innerHTML = "to " + BIG_VAL;
  }
  else if (hrange.substring(hrange.length - 4, hrange.length) == "8000"){
    oDTStckRange.innerHTML = "to " + hrange.substring(0,hrange.length - 4) +
                             "7FFF";
  }
  else{
     var i = 5;
     var append = "FFFF";
     while(hrange.charAt(hrange.length - i) == '0'){
       append = append + "F";
       i++;    
     } 
     oDTStckRange.innerHTML = "to " + hrange.substring(0,hrange.length - i) +
                               toHex("0x" + hrange.charAt(hrange.length - i) - 1) + 
                               append;
  }
}


//Convert a YYYY date to the number of days that have passed
function yearToDays(val){

  var retDays = 0;
  for (i = +val; i > BASE_YEAR; i--){
    if (isLeapYear(i - 1)){
       retDays += DAYS_LEAPYEAR;
    }
    else{
       retDays += DAYS_YEAR;
    } 
  }
  return retDays;
}


//Convert floating point number to a decimal
function floatToDec(val){
  
  //very small numbers are often in exp notation
  //and will cause problems with parseInt
  if (+val < 1){
    return 0;
  }
  return parseInt(val);
}

//Add zero padding to the front of a string
//value that is not at least the padded length
function addPad(val,pad){

  val = "" + val;
  var retStr = val;
  for (i = 0; i < (pad - val.length); i++){
    retStr = "0" + retStr;
  } 
  return retStr;
}

//Determine whether the year we are dealing with is a leap year
function isLeapYear(year){
  
  if ((year % 4 == 0) && (year % 100 != 0 || year % 400 == 0)){ 
    return true;
  }
  return false;
}

//Determine whether the input string is a valid hex string
function isValidHex(val){

  //validate each char
  for (i = 0; i < val.length; i++){
    switch (val.charAt(i)){
      case 'a':
      case 'b':
      case 'c':
      case 'd':
      case 'e':
      case 'f':
      case 'A':
      case 'B':
      case 'C':
      case 'D':
      case 'E':
      case 'F':
      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
         break;
      default:
	return false;
    } 
  }
  return true;
}


//Convert a decimal to hex.  Overflow is returned if the result
//is larger than a double word.
function toHex(val){

  var retval = "";
  while (true){
     retval = "" + toHexVal(val % 16) + retval;
     if (val < 16){
       if (retval.length > 16){
          return OVERFLOW;
       }
       else{
         return retval;
       }
     }
     val = parseInt(val / 16);
  } 
}

//Convert a value from 0 to 15 to the hex string equivalent
function toHexVal(val){

  switch(+val){
    case 1:
    case 2:
    case 3:
    case 4:
    case 5:
    case 6:
    case 7:
    case 8:
    case 9:
    case 0:
      return val;
      break;
    case 10:
      return "A";
      break;
    case 11:
      return "B";
      break;
    case 12:
      return "C";
      break;
    case 13:
      return "D";
      break;
    case 14:
      return "E";
      break;
    case 15:
      return "F";
      break;
    default:
      return "?";
  }
}

//Determine if the value passed is a valid integer.
//If max and min are different numbers, it will insure that
//the number is between these numbers inclusive.
function isValidInt(val,max,min){
 
  //we only want numbers...no decimals or negative signs
  for (i = 0; i < val.length; i++){
    if (val.charAt(i) > '9' || val.charAt(i) < '0'){
      return false;
    }
  }
  
  //are we out of bounds  
  if ((val.length > 0) && (+max != min) && ((+val > +max) || +val < min)){
    return false;
  } 
  return true;
}
